/**
  *  \file server/jsonin.h
  *  \brief Server: JSON input
  */
#ifndef C2SERVER_SERVER_JSONIN_H
#define C2SERVER_SERVER_JSONIN_H

#include "cpluslib/string.h"
#include "int/value.h"

class JsonTokenizer {
 public:
    enum Token {
        kError,
        kEOF,
        kString,
        kKeyword,
        kInt,
        kFloat,
        kStartArray,
        kEndArray,
        kStartHash,
        kEndHash,
        kComma,
        kColon
    };
    JsonTokenizer(string_t text);

    Token readNext();

    Token getCurrent() const;
    double getFloatValue() const;
    int32_t getIntValue() const;
    string_t getStringValue() const;

 private:
    Token current_token;
    double current_float;       /* kFloat */
    int32_t current_int;        /* kInt */
    string_t current_string;    /* kKeyword, kString */

    string_t text;
    string_t::size_type pos;
};


IntValue* readTree(JsonTokenizer& tok);



inline JsonTokenizer::Token
JsonTokenizer::getCurrent() const
{
    return current_token;
}

inline double
JsonTokenizer::getFloatValue() const
{
    return current_float;
}

inline int32_t
JsonTokenizer::getIntValue() const
{
    return current_int;
}

inline string_t
JsonTokenizer::getStringValue() const
{
    return current_string;
}

#endif
