/**
  *  \file server/objout.h
  *  \brief Server: Object Output
  */
#ifndef C2SERVER_SERVER_OBJOUT_H
#define C2SERVER_SERVER_OBJOUT_H

class JsonWriter;

class ServerObjectWriter {
 public:
    virtual ~ServerObjectWriter();
    virtual void write(JsonWriter&) = 0;
    virtual bool isSame(ServerObjectWriter* other) = 0;
};

class ServerSingularWriter : public ServerObjectWriter {
 public:
    bool isSame(ServerObjectWriter* other);
};

class ServerShipxyWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerShipWriter : public ServerObjectWriter {
    int sid;
 public:
    ServerShipWriter(int sid);
    void write(JsonWriter& w);
    bool isSame(ServerObjectWriter* other);
};

class ServerPlanetxyWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerPlanetWriter : public ServerObjectWriter {
    int pid;
 public:
    ServerPlanetWriter(int pid);
    void write(JsonWriter& w);
    bool isSame(ServerObjectWriter* other);
};

class ServerMainWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerPlayerWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerHullWriter : public ServerObjectWriter {
    int hid;
 public:
    ServerHullWriter(int hid);
    void write(JsonWriter& w);
    bool isSame(ServerObjectWriter* other);
};

class ServerTorpWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerBeamWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerEngineWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerStormWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerMinefieldWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerUfoWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerMessageWriter : public ServerObjectWriter {
    int index;
 public:
    ServerMessageWriter(int index);
    void write(JsonWriter& w);
    bool isSame(ServerObjectWriter* other);
};

class ServerTruehullWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

class ServerVcrWriter : public ServerSingularWriter {
 public:
    void write(JsonWriter& w);
};

#endif
